package java.util.zip;

import java.util.Date;
import java.util.concurrent.TimeUnit;
import java.nio.file.attribute.FileTime;

class ZipUtils
{
    private static final long WINDOWS_EPOCH_IN_MICROSECONDS = -11644473600000000L;
    
    ZipUtils() {
    }
    
    public static final FileTime winTimeToFileTime(long n) {
        return FileTime.from(n / 10L - 11644473600000000L, TimeUnit.MICROSECONDS);
    }
    
    public static final long fileTimeToWinTime(FileTime fileTime) {
        return (fileTime.to(TimeUnit.MICROSECONDS) + 11644473600000000L) * 10L;
    }
    
    public static final FileTime unixTimeToFileTime(long value) {
        return FileTime.from(value, TimeUnit.SECONDS);
    }
    
    public static final long fileTimeToUnixTime(FileTime fileTime) {
        return fileTime.to(TimeUnit.SECONDS);
    }
    
    private static long dosToJavaTime(long n) {
        return new Date((int)((n >> 25 & 0x7FL) + 80L), (int)((n >> 21 & 0xFL) - 1L), (int)(n >> 16 & 0x1FL), (int)(n >> 11 & 0x1FL), (int)(n >> 5 & 0x3FL), (int)(n << 1 & 0x3EL)).getTime();
    }
    
    public static long extendedDosToJavaTime(long n) {
        return dosToJavaTime(n) + (n >> 32);
    }
    
    private static long javaToDosTime(long date) {
        Date date2 = new Date(date);
        int n = date2.getYear() + 1900;
        if (n < 1980) {
            return 2162688L;
        }
        return (long)(n - 1980 << 25 | date2.getMonth() + 1 << 21 | date2.getDate() << 16 | date2.getHours() << 11 | date2.getMinutes() << 5 | date2.getSeconds() >> 1);
    }
    
    public static long javaToExtendedDosTime(long n) {
        if (n < 0L) {
            return 2162688L;
        }
        long javaToDosTime = javaToDosTime(n);
        return (javaToDosTime != 2162688L) ? (javaToDosTime + (n % 2000L << 32)) : 2162688L;
    }
    
    public static final int get16(byte[] array, int n) {
        return Byte.toUnsignedInt(array[n]) | Byte.toUnsignedInt(array[n + 1]) << 8;
    }
    
    public static final long get32(byte[] array, int n) {
        return ((long)get16(array, n) | (long)get16(array, n + 2) << 16) & 0xFFFFFFFFL;
    }
    
    public static final long get64(byte[] array, int n) {
        return get32(array, n) | get32(array, n + 4) << 32;
    }
}
